; ***************************************************************
; * Kuppelsteuerung einer Teleskopkuppel                        *

; * mittels Befehlen einer Teleskopsoftware                     *

; * ber ein standardisiertes Protokoll                         *

; * per V.24 ( RS232 ) Schnittstelle                            *

; * Auftraggeber: www.colouruniverse.de                         *

; * (C)2005 by mad.dax@t-online.de Letzte nderung: 28.11.05    *

; * Status: Hyst und Teiler ber Uart ( sollte funzen )         *

; *         Testbetrieb ohne Steuerung der Kuppelabdeckung      *
; ***************************************************************
;
; Hardware Anforderungen:

; Contoller AVR 2333 
;
; Standard Belegung
;
;  1  RESET                    28  PC5  bzw. ADC5
;  2  PD0   bzw. RXD           27  PC4  bzw. ADC4   
;  3  PD1   bzw. TXD           26  PC3  bzw. ADC3   
;  4  PD2   bzw. INT0          25  PC2  bzw. ADC2   
;  5  PD3   bzw. INT1          24  PC1  bzw. ADC1   
;  6  PD4   bzw. T0            23  PC0  bzw. ADC0   
;  7  VCC                      22  AGND             
;  8  GND                      21  AREF             
;  9  XTAL1                    20  AVCC             
; 10  XTAL2                    19  PB5  bzw. SCK    
; 11  PD5   T1                 18  PB4  bzw. MISO   
; 12  PD6   AIN0               17  PB3  bzw. MOSI   
; 13  PD7   AIN1               16  PB2  bzw. SS     
; 14  PB0   ICP                15  PB1  bzw. OC1
;
; Dome Belegung
;
;  1  RESET                    28  PC5  TX Data Indicator
;  2  RXD   RS232              27  PC4  RX Data Indicator
;  3  TXD   RS232              26  PC3  Rotate Left 
;  4  INT0  Azimut             25  PC2  Rotate Right Clockwise 
;  5  INT1  Azi 0000           24  PC1  Close Shut  
;  6  PD4   Azi Dir            23  PC0  Open Shut   
;  7  VCC                      22  AGND             
;  8  GND                      21  AREF             
;  9  XTAL1                    20  AVCC             
; 10  XTAL2                    19  PB5  SCK bzw. LongAZI bzw. RTS  
; 11  PD5                      18  PB4  MISO   
; 12  PD6   Shut Open          17  PB3  MOSI bzw. StartVal bzw. DTR  
; 13  PD7   Shut Closed        16  PB2  
; 14  PB0                      15  PB1  


; Software Funktionen:
;
.NOLIST
.INCLUDE "2333def.inc"
;
; Konstanten
;
.equ RAMRX = 0x60 	;Start-Adresse-SRAM
.equ RAMCP = 0x75 	;Arbeits-Adresse-SRAM ( Kopie von RAMRX )
.equ RAMTX = 0x90 	;Mittel-Adresse-SRAM
.equ Quartz = 4000000 	;fr 4 MHz 
.equ Baud = 19200 	;UART-Baudrate
.equ UBRRVAL = Quartz / (Baud * 16) - 1
.equ RX2ND = '#'	;zweites Zeichen
.equ RX1ST = 'd'	;Startzeichen
.equ SLEN = 8		;Stringlnge
.equ DLEN = 9		;Datenlnge
.equ RXLEN = 7		;Datenlnge
.equ AZLEN = 5		;AZIMUT Lnge
.equ AZILEN = 3		;AZIMUT Lnge



; Konstanten FlagReg
;
.equ MsgOK = 7		;Antwort auf korrekte Anfrage
.equ GetAZ = 6		;Anfrage Stellung Kuppel erkannt 
.equ GetSH = 5		;Anfrage Stellung Klappe erkannt 
.equ OpSH = 4		;Befehl ffnen erkannt 
.equ ClSH = 3		;Befehl Schlieen erkannt
.equ TValid = 2		;Telegramm gltig
.equ TNext = 1		;Telegramm erkannt
.equ TStart = 0		;Telegrammstart bei 1

; Konstanten FlagReg2
;
;equ  = 7		;
;equ  = 6		;
;equ  = 5		;
;equ  = 4		;
;equ  = 3		;
;equ  =2		;
.equ GetMUL = 1		;Anfrage auf Multiplikator erkannt
.equ GetHYS = 0		;Anfrage auf Hysterese erkannt

;Konstanten Ein- Ausgnge

.equ ShutOpen = 6
.equ ShutClosed = 7
.equ OpenShut = 0
.equ CloseShut = 1
.equ RotateRight = 2
.equ RotateLeft = 3
.equ AziDir = 4
.equ AziReset = 3
.equ StartLeft = 2
.equ StartRight = 1
.equ RXDat = 4
.equ TXDat = 5
.equ LongAZI = 5
.equ StartVal = 3


; Benutzte Register
;
.def STRING = R0     ;direkter Speicherzugriff
.def WREEP = R1	     ;EEprom Schreibzhler
.def IMAXLO = R2     ;Impulszhler MAX
.def IMAXHI = R3     ;Impulszhler MAX
.def HYSTLO = R4     ;Hysteresewert Promille * 18
.def HYSTHI = R5     ;Hysteresewert Promille * 18 	
.def TeilerLO = R6   ;Teiler fr Winkelzhler
.def TeilerHI = R7   ;Teiler fr Winkelzhler			     
.def TEMP7 = R8      ;Temp fr Division	  
.def TEMP8 = R9      ;Temp fr Division
.def AziLO = R10     ;Azimut Soll LO-Byte
.def AziHI = R11     ;Azimut Soll Hi-Byte
.def AziVALLO = R12  ;Azimut IST LO-Byte
.def AziVALHI = R13  ;Azimut IST Hi-Byte
.def STIMERHi = R14  ;Soft-Timer mit Interupt von Timer1
.def STIMERLo = R15  ;Soft-Timer mit Interupt von Timer1
.def TEMP = R16      ;TEMP Register fr Initialisierung und Schmierregister
.def TEMP1 = R17     ;TEMP Register fr Hauptprogramm und Initialisierung	
.def TEMP2 = R18     ;TEMP Register fr Hauptprogramm
.def TEMP3 = R19     ;TEMP Register fr Hauptprogramm
.def TEMP4 = R20     ;TEMP Register fr Hauptprogramm
.def TEMP5 = R21     ;TEMP Register fr Hauptprogramm
.def TEMP6 = R22     ;TEMP Register fr Hauptprogramm
.def FlagReg = R23   ;Flagregister 	
.def FlagRg2 = R24   ;Flagregister2	
.def DATA = R25  
;def Reserviert = R26 
;def Reserviert = R27
.def ByteCNT = R28
.def TByteCNT = R29
;def Reserviert = R30
;def Reserviert = R31
;
;
; Code beginnt hier
;
.CSEG
.ORG $0000
;
; Reset- und Interrupt-Vektoren
;
         rjmp Start           ; Reset-vector
         rjmp AziCNT          ; External Interrupt Request 0
         rjmp AziSync         ; External Interrupt Request 1
         reti                 ; Timer/Counter1 Capture Event
         reti                 ; Timer/Counter1 Compare Match
         reti                 ; Timer/Counter1 Overflow
         reti                 ; Timer/Counter0 Overflow
         reti                 ; SPI Serial Transfer Complete
         rjmp Receive         ; Uart Rx char available
         reti                 ; Uart Tx data register empty
         reti                 ; Uart Tx complete
         reti                 ; EEProm Ready
         reti                 ; Analog comparator
;
;
; **************** Ende der Interrupt Service Routinen *********************

;
; **************** Makros fr oft gebrauchte Programmteile ***************** 
;
.macro RXLum

	sbic PortC, RXDat
	rjmp RXRES
	sbi PortC, RXDat
	rjmp RXEnd
RXRES:	cbi PortC, RXDat
RXEnd:
.endmacro

.macro TXLum

	sbic PortC, TXDat
	rjmp TXRES
	sbi PortC, TXDat
	rjmp TXEnd
TXRES:	cbi PortC, TXDat
TXEnd:
.endmacro

.macro PUSHTmp
         push TEMP2
         push TEMP1           ;
         push TEMP            ;
         in TEMP,SREG         ;
         push TEMP            ;
.endmacro 

.macro POPTmp
         pop TEMP
         out SREG,TEMP
         pop TEMP             ;
         pop TEMP1            ;
         pop TEMP2
.endmacro

.macro PUSHSREG
         in TEMP,SREG         ;
         push TEMP            ;
.endmacro 

.macro POPSREG
         pop TEMP
         out SREG,TEMP
.endmacro
;
; *****************Verschiedene Unterprogramme******************************
;

writeEEPROM:
         sbic EECR,EEWE       ; wird momentan noch geschrieben ?
         rjmp PC-1            ; Schleife 
         out EEAR,TEMP2       ; Erfasse EEProm Schreibadresse
         out EEDR,TEMP1       ; Erfasse EEProm Schreibdaten 
         cli                  ; alle Interrupts ausschalten
         sbi EECR,EEMWE       ; Setze EEProm Master-Schreibbit 
         sbi EECR,EEWE        ; Setze EEProm Schreib-Befehl
         sei                  ; alle Interrupts wieder einschalten
         ret                  ; Rcksprung
;
;
readEEPROM:
         sbic EECR, EEWE      ; wird noch geschrieben ?
         rjmp PC-1            ; Schleife
         out EEAR, TEMP       ; Erfasse EEProm-Leseadresse
         sbi EECR, EERE       ; Setze EEProm-Lesebefehl 
         in TEMP1, EEDR       ; Lese EEprom-Daten
         ret                  ; Rcksprung

AziCNT: ;Azimut Zhler Interuptgesteuert
	 
	 sbis PinD, AziDir		;linkslauf ?
	 rjmp Left
	 TXLum
	 add AziValLO, TeilerLO
	 adc AziValHI, TeilerHI
	 
	 cp AziValHI, IMAXHI
	 brlo AziCNTEnd
	 breq AziINCEQ
	 clr AziValHI			; berlauf
	 clr AziValLO
	 rjmp AziCNTEnd
	 
AziINCEQ:cp AziValLO, IMAXLO
	 brlo AziCNTEnd
	 breq AziCNTEnd
	 clr AziValHI			; berlauf
	 clr AziValLO
	 rjmp AziCNTEnd
	 	 
Left:	 sub AziValLO, TeilerLO
	 sbc AziValHI, TeilerHI
         RXLum
	 cp AziValHI, IMAXHI
	 brlo AziCNTEnd
	 breq AziDECEQ
	 mov AziValHI, IMAXHI		; berlauf
	 mov AziValLO, IMAXLO
	 rjmp AziCNTEnd
	 
AziDECEQ:cp AziValLO, IMAXLO
	 brlo AziCNTEnd
	 breq AziCNTEnd
	 mov AziValHI, IMAXHI		; berlauf
	 mov AziValLO, IMAXLO
	 	 
AziCNTEnd:
	 reti                 ; Rcksprung und alle Interrupts wieder einschalten


AziSync:;Azimut Synchronisation auf Sd Interuptgesteuert
	;RXLum
	;PUSHTmp
	;rcall SafeAZI
	;POPTmp
	clr AziValLO
	clr AziValHI
	;rcall Azisend
	;cbi PortC, RotateRight
	;cbi PortC, RotateLeft
	reti                 ; Rcksprung und alle Interrupts wieder einschalten

;Commands from Telecope
;db "getshut",1;(Get Shutterstatus)      64,23, 67,65,74,73,68,75,74
;db "opeshut",2;(open shutter)           64,23, 6F,70,65,73,68,75,74
;db "closhut",3;(schliee Kuppelspalt)   64,23, 63,6C,6F,73,68,75,74
;db "getazim",4;(get Azimutposition)     64,23, 67,65,74,61,7A,69,6D
;db "azi0000",5;(South)                  64,23, 61,7A,69,30,30,30,30
;db "teil000",6;(Teilerwert)             64,23, 74,65,6C,30,30,30,30

Receive:;Empfangsroutine Interuptgesteuert
	 PUSHTmp	      ; Temp Register retten
	 in TEMP, UDR         ; lade Temp mit UART-Daten
	 
	 ;RXLum
	 	          
         sbic UCSRA, FE	      ; framing error
         rjmp RXFehler
                  
         sbic UCSRA, OR	      ; overrun
         rjmp RXFehler
         
         sbrc Flagreg, TStart ; wenn Telegrammstart erhalten dann
         rjmp RXNext	      ; springe zum zweiten Zeichen	
         cpi TEMP, RX1ST      ; Vergleiche RX-Daten mit Start d
         brne RXFehler	      ; wenn ungleich dann Fehler	
         sbr FlagReg, (1<<TStart); setze erstes Zeichen OK
         POPTmp               ; Rcksichern Register
         reti                 ; Rcksprung und alle Interrupts wieder einschalten
	 
RXNext:	 sbrc Flagreg, TNext  ; wenn Telegramm NEXT erhalten dann
         rjmp RXData	      ; springe zum Datenempfang	
         cpi TEMP, RX2ND      ; Vergleiche RX-Daten mit Start #
         brne RXFehler	      ; wenn ungleich dann Fehler	
         sbr FlagReg, (1<<TNext); setze Zweites Zeichen OK
         ldi XH, HIGH(RAMRX)  ; Inizialisiere X Pointer High-Adresse mit RAM-Start-Adresse ( Datenempfang )
         ldi XL, LOW(RAMRX)   ; Inizialisiere X Pointer Low-Adresse mit RAM-Start-Adresse ( Datenempfang ) 
         clr ByteCNT	      ; Initialisiere Bytezhler
         POPTmp               ; Rcksichern Register
         reti                 ; Rcksprung und alle Interrupts wieder einschalten
          
RXData:  st X+ ,TEMP           ; speichere UART-Daten im RAM
         inc ByteCNT          ; zhle empfangene Daten-Bytes
	 cpi ByteCNT, RXLEN   ; Vergleiche auf Telegrammende
	 breq RXEnde	      ; und springe zur Auswertung
	 POPTmp               ; Rcksichern Register
         reti                 ; Rcksprung und alle Interrupts wieder einschalten
         
RXEnde:  sbr Flagreg, (1<<TValid) ; Telegramm gltig	
                   
RXFehler:cbr Flagreg, (1<<TSTART) + (1<<TNext)
	 POPTmp               ; Rcksichern Register 
	 reti                 ; Rcksprung und alle Interrupts wieder einschalten
	 
     
Transmit:;Senderoutine Text
         ;TXLUM
         lpm		      ; Hole String
         
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, STRING      ; Telegramm an UART senden
         
         adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler
	 cpi TByteCNT, DLEN   ; Vergleiche auf Telegrammende
	 brne Transmit	      ; Schleife	      
	 ret                  ; und springe zurck zum Hauptprogramm
	
Transaz:;Senderoutine Azimut
         ;TXLUM
         lpm		      ; Hole String
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, STRING      ; Telegramm an UART senden
         adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler
	 cpi TByteCNT, AZLEN ; Vergleiche auf Telegrammende
	 brne Transaz	      ; Schleife
	 ;Rckrechen Azimut	      
	 mov TEMP,AziVALLO   ; Azimut fr Division sichern
	 mov TEMP1,AziVALHI
	 
	 ldi TEMP2, LOW(18)   ; Division durch 18
	 ldi TEMP3, HIGH(18)
	 rcall div16u	      ; Division aufrufen
	 	 
	 ldi TEMP2, LOW(1000) ; Vergleich auf 1000er
	 ldi TEMP3, HIGH(1000)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, LOW(100)  ; Vergleich auf 100er
	 ldi TEMP3, HIGH(100)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, LOW(10)   ; Vergleich auf 10er
	 ldi TEMP3, HIGH(10)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TEMP, TEMP2      ; sende Rest
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TEMP        ; Telegramm an UART senden
         
         ret                  ; und springe zurck zum Hauptprogramm
         
Transazval:;Senderoutine Azimut Zhlerwert
                   
         ldi TEMP, RX2ND
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TEMP        ; Telegramm an UART senden
                              
         mov TEMP, AziValLO   ; Azimut fr kopieren
	 mov TEMP1, AziValHI
	 
	 ldi TEMP2, LOW(10000) ; Vergleich auf 10000er
	 ldi TEMP3, HIGH(10000)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         	 	 	 
	 ldi TEMP2, LOW(1000) ; Vergleich auf 1000er
	 ldi TEMP3, HIGH(1000)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, LOW(100)  ; Vergleich auf 100er
	 ldi TEMP3, HIGH(100)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, LOW(10)   ; Vergleich auf 10er
	 ldi TEMP3, HIGH(10)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TEMP, TEMP2      ; sende Rest
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TEMP        ; Telegramm an UART senden
         
         ret                  ; und springe zurck zum Hauptprogramm

Transstartval:;Senderoutine Teilerwert und Hysterese
         ldi TEMP, RX2ND
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TEMP        ; Telegramm an UART senden
         
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TeilerHI    ; Telegramm an UART senden
         
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TeilerLO    ; Telegramm an UART senden
         
         ldi TEMP, RX2ND
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TEMP        ; Telegramm an UART senden
         
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, HYSTHI      ; Telegramm an UART senden
         
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, HYSTLO      ; Telegramm an UART senden
         
         ldi TEMP, RX2ND
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TEMP        ; Telegramm an UART senden
         
         ret                  ; und springe zurck zum Hauptprogramm 
         
TransHYS:;Senderoutine Hysterese
         ;TXLUM
         lpm		      ; Hole String
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, STRING      ; Telegramm an UART senden
         adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler
	 cpi TByteCNT, AZLEN  ; Vergleiche auf Telegrammende
	 brne TransHYS	      ; Schleife
	 ;Rckrechen Hysterese	      
	 mov TEMP,HYSTLO      ; Hysterese fr Division sichern
	 mov TEMP1,HYSTHI
	 
	 ldi TEMP2, LOW(18)   ; Division durch 18
	 ldi TEMP3, HIGH(18)
	 rcall div16u	      ; Division aufrufen
	 	 
	 ldi TEMP2, LOW(1000) ; Vergleich auf 1000er
	 ldi TEMP3, HIGH(1000)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, LOW(100)  ; Vergleich auf 100er
	 ldi TEMP3, HIGH(100)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, LOW(10)   ; Vergleich auf 10er
	 ldi TEMP3, HIGH(10)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TEMP, TEMP2      ; sende Rest
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TEMP        ; Telegramm an UART senden
         
         ret                  ; und springe zurck zum Hauptprogramm

TransMUL:;Senderoutine Multiplikator
         ;TXLUM
         lpm		      ; Hole String
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, STRING      ; Telegramm an UART senden
         adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler
	 cpi TByteCNT, AZLEN  ; Vergleiche auf Telegrammende
	 brne TransMUL	      ; Schleife
	     
	 mov TEMP,TeilerLO      ; Multiplikator sichern
	 mov TEMP1,TeilerHI
	 
	 ldi TEMP2, LOW(1000) ; Vergleich auf 1000er
	 ldi TEMP3, HIGH(1000)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, LOW(100)  ; Vergleich auf 100er
	 ldi TEMP3, HIGH(100)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, LOW(10)   ; Vergleich auf 10er
	 ldi TEMP3, HIGH(10)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TEMP, TEMP2      ; sende Rest
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TEMP        ; Telegramm an UART senden
         
         ret                  ; und springe zurck zum Hauptprogramm         
        
         
Mulsend: clr TByteCNT	        ; Initialisiere Bytezhler
	 ldi ZH, HIGH(2*AXMUL)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse
         ldi ZL, LOW(2*AXMUL)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse
         rcall TransMUL
         TXlum
         cbr FlagRg2, (1<<GetMUL)
         ret       

Hyssend: clr TByteCNT	        ; Initialisiere Bytezhler
	 ldi ZH, HIGH(2*AXHYS)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse
         ldi ZL, LOW(2*AXHYS)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse
         rcall TransHys
         RXlum
         cbr FlagRg2, (1<<GetHYS)
         ret       

TransVAL:;Senderoutine empfangener Aziwert 
         ;TXLUM
         lpm		      ; Hole String
         sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, STRING      ; Telegramm an UART senden
         adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler
	 cpi TByteCNT, AZLEN  ; Vergleiche auf Telegrammende
	 brne TransVAL	      ; Schleife
	 ;Rckrechen Hysterese	      
	 mov TEMP,AZILO      ; Wert fr Division sichern
	 mov TEMP1,AZIHI
	 
	 ldi TEMP2, LOW(18)   ; Division durch 18
	 ldi TEMP3, HIGH(18)
	 rcall div16u	      ; Division aufrufen
	 	 
	 ldi TEMP2, LOW(1000) ; Vergleich auf 1000er
	 ldi TEMP3, HIGH(1000)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, LOW(100)  ; Vergleich auf 100er
	 ldi TEMP3, HIGH(100)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, LOW(10)   ; Vergleich auf 10er
	 ldi TEMP3, HIGH(10)
	 rcall Bin2ToDigit
	 
	 ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TByteCNT, TEMP2
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TByteCNT    ; Telegramm an UART senden
         
         ldi TEMP2, 48        ; addiere 48 fr ASCII Code
	 add TEMP, TEMP2      ; sende Rest
	 sbis UCSRA,UDRE      ; Warten, bis UDR bereit ist
         rjmp PC-1            ; Schleife
         out UDR, TEMP        ; Telegramm an UART senden
         
         ret                  ; und springe zurck zum Hauptprogramm
                 
SafeAZI: ;nicht verwendet 
	ldi TEMP, 8
	inc WREEP
	cp WREEP, TEMP
	brsh SyncN
	mov WREEP, TEMP
SyncN:	mov TEMP2, WREEP
	mov TEMP1, AziValHI
	rcall writeeeprom
	inc WREEP
	cp WREEP, TEMP
	brsh SyncM
	mov WREEP, TEMP
SyncM:	mov TEMP2, WREEP
	mov TEMP1, AziValLO
	rcall writeeeprom
	
	ret                  ; und springe zurck zum Hauptprogramm
	
; 8x8 Bit Unsigned Multiplication von ATMEL

;def	TEMP1	=r16		;multiplicand
;def	TEMP2	=r17		;multiplier
;def	TEMP2	=r17		;result Low byte
;def	TEMP3	=r18		;result High byte


MUL8X8:	 clr	TEMP3		;clear result High byte
	 ldi	ByteCNT,8	;init loop counter
	 lsr	TEMP2		;rotate multiplier
	
MULSTART:
	 brcc	MULMORE		;carry set 
	 add 	TEMP3,TEMP1	;add multiplicand to result High byte
MULMORE:
	 ror	TEMP3		;rotate right result High byte
	 ror	TEMP2		;rotate right result L byte and multiplier
	 dec	ByteCNT		;decrement loop counter
	 brne	MULSTART	;if not done, loop more
	 ret
	
;***************************************************************************
;*
;* "mpy16u" - 16x16 Bit Unsigned Multiplication
;*
;* This subroutine multiplies the two 16-bit register variables 
;* mp16uH:mp16uL and mc16uH:mc16uL.
;* The result is placed in m16u3:m16u2:m16u1:m16u0.
;*  
;* Number of words	:14 + return
;* Number of cycles	:153 + return
;* Low registers used	:None
;* High registers used  :7 (mp16uL,mp16uH,mc16uL/m16u0,mc16uH/m16u1,m16u2,
;*                          m16u3,mcnt16u)	
;*
;***************************************************************************

;***** Subroutine Register Variables

.def	mc16uL	=r16		;multiplicand low byte   TEMP
.def	mc16uH	=r17		;multiplicand high byte  TEMP1
.def	mp16uL	=r18		;multiplier low byte     TEMP2
.def	mp16uH	=r19		;multiplier high byte    TEMP3
.def	m16u0	=r18		;result byte 0 (LSB)     TEMP2
.def	m16u1	=r19		;result byte 1           TEMP3
.def	m16u2	=r20		;result byte 2           TEMP4
.def	m16u3	=r21		;result byte 3 (MSB)     TEMP5
.def	mcnt16u	=r22		;loop counter            TEMP6

;***** Code

mpy16u:	clr	m16u3		;clear 2 highest bytes of result
	clr	m16u2
	ldi	mcnt16u,16	;init loop counter
	lsr	mp16uH
	ror	mp16uL

m16u_1:	brcc	noad8		;if bit 0 of multiplier set
	add	m16u2,mc16uL	;add multiplicand Low to byte 2 of res
	adc	m16u3,mc16uH	;add multiplicand high to byte 3 of res
noad8:	ror	m16u3		;shift right result byte 3
	ror	m16u2		;rotate right result byte 2
	ror	m16u1		;rotate result byte 1 and multiplier High
	ror	m16u0		;rotate result byte 0 and multiplier Low
	dec	mcnt16u		;decrement loop counter
	brne	m16u_1		;if not done, loop more
	ret
	
;***************************************************************************
;*
;* "div16u" - 16/16 Bit Unsigned Division
;*
;* This subroutine divides the two 16-bit numbers 
;* "dd8uH:dd8uL" (dividend) and "dv16uH:dv16uL" (divisor). 
;* The result is placed in "dres16uH:dres16uL" and the remainder in
;* "drem16uH:drem16uL".
;*  
;* Number of words	:19
;* Number of cycles	:235/251 (Min/Max)
;* Low registers used	:2 (drem16uL,drem16uH)
;* High registers used  :5 (dres16uL/dd16uL,dres16uH/dd16uH,dv16uL,dv16uH,
;*			    dcnt16u)
;*
;***************************************************************************

;***** Subroutine Register Variables

.def	drem16uL=r8	;TEMP7
.def	drem16uH=r9	;TEMP8
.def	dres16uL=r16	;TEMP
.def	dres16uH=r17	;TEMP1
.def	dd16uL	=r16	;TEMP
.def	dd16uH	=r17	;TEMP1
.def	dv16uL	=r18	;TEMP2
.def	dv16uH	=r19	;TEMP3
.def	dcnt16u	=r20	;TEMP4

;***** Code

div16u:	clr	drem16uL	;clear remainder Low byte
	sub	drem16uH,drem16uH;clear remainder High byte and carry
	ldi	dcnt16u,17	;init loop counter
d16u_1:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	dec	dcnt16u		;decrement counter
	brne	d16u_2		;if done
	ret			;return
d16u_2:	rol	drem16uL	;shift dividend into remainder
	rol	drem16uH
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_3		;if result negative
	add	drem16uL,dv16uL	;restore remainder
	adc	drem16uH,dv16uH
	clc			;clear carry to be shifted into result
	rjmp	d16u_1		;else
d16u_3:	sec			;set carry to be shifted into result
	rjmp	d16u_1


; ********************************************************
; * Routinen zur Zahlumwandlung, Version 0.1 Januar 2002 *
; * (C)2002 by info@avr-asm-tutorial.net                 *
; ********************************************************

Bin2ToDigit:
	clr TByteCNT      ; Zhler auf Null
Bin2ToDigita:
	cp TEMP1,TEMP3    ; Vergleiche MSBs miteinander
	brcs Bin2ToDigitc ; MSB Binrzahl kleiner, fertig
	brne Bin2ToDigitb ; MSB Binrzahl grer, subtrahiere
	cp TEMP,TEMP2     ; MSB gleich, vergleiche LSBs
	brcs Bin2ToDigitc ; LSB Binrzahl kleiner, fertig
Bin2ToDigitb:
	sub TEMP,TEMP2    ; Subtrahiere LSB Dezimalzahl
	sbc TEMP1,TEMP3   ; Subtrahiere Carry und MSB
	inc TByteCNT      ; Erhhe den Zhler
	rjmp Bin2ToDigita ; Weiter vergleichen/subtrahieren
Bin2ToDigitc:
	ret ; zurck	 	 
;
; **************** Ende der Unterprogramme *********************
;
; ******************** Hauptprogramm ***************************
;
; Hauptprogramm beginnt hier
;
Start:   
      
        ldi TEMP,0b10011000   ;RX Complete, RX und TX aktivieren
        out UCSRB, TEMP       ;UART Control und Status       
                          
        ;ldi TEMP,0b10000010  ;Timer 0 und Timer 1 Overflow Interrupt gesetzt
        ;out TIMSK, TEMP      ;Timer/Counter Interupt Mask
         
        ;ldi TEMP,0b00000011  ;Timer 0 Prescaler auf 64
        ;out TCCR0, TEMP      ;Timer/Counter 0 Control
        
        ldi TEMP,0b11000000   ;Interupt 0 und 1 enable
        out GIMSK, TEMP       ;General Interupt Mask
        
        ldi TEMP,0b00001010   ;Interupt 0 und 1 auf fallende Flanke
        out MCUCR, TEMP       ;MCU Control Register
        
        ldi TEMP,0b00000000   ;Port B 
        out DDRB, TEMP        ;als Eingang
        
        ldi TEMP,0b11111111   ;Port C 
        out DDRC, TEMP        ;als Ausgang
        
        ldi TEMP,0b00000000   ;Port D 
        out DDRD, TEMP        ;als Eingang
        
;Anlauf nach Programmstart
        
AZLEFT:	sbi PortC, RotateLeft
	sbi PortC, RotateRight
	rjmp AZINIT

AZRIGHT:sbi PortC, RotateRight
	cbi PortC, RotateLeft
	
AZINIT: sbic PinD, AziReset ; Reset bei 0
        ;rjmp PC-1
        

        ldi TEMP,RAMEND      ; lade Low-Byte der RAM-Ende-Adresse nach TEMP
        out SPL,TEMP         ; Inizialisiere Stack Pointer Low-Adresse
        
        ;ldi TEMP,high(RAMEND); lade High-Byte der RAM-Ende-Adresse nach TEMP
        ;out SPH, TEMP        ; Inizialisiere Stack Pointer High-Adresse
                     
        clr AziLO
	clr AziHI
	clr AziVALLO
	clr AziVALHI
	clr Flagreg
	
;Init Impulszhler	
	ldi TEMP, 0x1F
	mov IMAXLO, TEMP
	ldi TEMP, 0xFD
	mov IMAXHI, TEMP

;Teiler aus EEPROM laden

         ldi TEMP , 0	      ; Teileradresse
         rcall readEEPROM
         mov TeilerHI, TEMP1
         
         ldi TEMP , 1	      ; Teileradresse
         rcall readEEPROM
         mov TeilerLO, TEMP1
         
;Hysterse aus EEPROM laden

         ldi TEMP , 2	      ; HystereseAdresse
         rcall readEEPROM
         mov HYSTHI, TEMP1
         
         ldi TEMP , 3	      ; HystereseAdresse
         rcall readEEPROM
         mov HYSTLO, TEMP1          
 
;UART initialisieren

	 ldi TEMP, UBRRVAL
         out UBRR, TEMP         ; setze Baudrate
	 	 
	 ldi  TEMP1,0b11000000	; lsche externen Interupt 0 und 1
	 out  GIFR,TEMP1
	 
	 ldi  TEMP1,0b11000000	; lsche UART Interups
	 out  UCSRA,TEMP1
	 
	 ldi TEMP, 8
	 mov WREEP, TEMP
	 	 	 
	 sei                  ; Interrupts generell aktivieren        

;***********Hauptprogrammschleife********************************************
	
	clr AziValLO
	clr AziValHI
        cbi PortC, RotateLeft
        cbi PortC, RotateRight
        rcall Azisend 
Main:

 
	 sbrc Flagreg,MsgOK		;Antwort auf korrekte Anfrage
	 rcall AMsgOK
	 
	 sbrc Flagreg,GetAZ		;Anfrage Stellung Kuppel erkannt
	 rcall Azisend
	 
	 sbrc FlagRg2,GetMUL		;Anfrage auf Multiplikator erkannt
	 rcall Mulsend
	 
	 sbrc FlagRg2,GetHYS		;Anfrage auf Hysterese erkannt
	 rcall Hyssend
	  
	 sbrc Flagreg,GetSH		;Anfrage Stellung Klappe erkannt
	 rcall AGetSH
	 	  
	 sbrc Flagreg,OpSH		;Befehl ffnen erkannt
	 rcall AOpSH
	  
	 sbrc Flagreg,ClSH		;Befehl Schlieen erkannt
	 rcall AClSH
	 
	 sbrc Flagreg,TValid		;Gltiges Telegramm
	 rcall AValid
	 
	 	 
;Jetzt aber zur Kuppelsteuerung !

        mov TEMP3, AziLO
        mov TEMP4, AziHI
        mov TEMP5, AziVALLO
        mov TEMP6, AziVALHI
                       
        cp AziHI,AziVALHI  ; Vergleiche MSBs miteinander
	brlo AZLO          ; MSB Binrzahl kleiner, fertig
	brne AZHI          ; MSB Binrzahl grer, fertig
	cp AziLO,AziVALLO  ; MSB gleich, vergleiche LSBs
	brlo AZLO          ; LSB Binrzahl kleiner, fertig
	
AZHI:   ;Soll grer Ist
	sub TEMP3,TEMP5    ; Subtrahiere LSB Dezimalzahl
	sbc TEMP4,TEMP6    ; Subtrahiere Carry und MSB
	
	mov TEMP5,Hystlo     ; Abweichung * 18
	mov TEMP6,Hysthi     ; Abweichung * 18
	
	cp TEMP4,TEMP6    ; Vergleiche MSBs miteinander
	brlo STOP   	  ; MSB Binrzahl kleiner, fertig
	brne AZHIW        ; MSB Binrzahl grer, fertig
	cp TEMP3,TEMP5    ; MSB gleich, vergleiche LSBs
	brlo STOP         ; LSB Binrzahl kleiner, fertig
	breq STOP

AZHIW:	ldi TEMP5, LOW(1800*18)
	ldi TEMP6, HIGH(1800*18)
	
	cp TEMP4,TEMP6    ; Vergleiche MSBs miteinander
	brlo RECHTS   	  ; MSB Binrzahl kleiner, fertig
	brne LINKS        ; MSB Binrzahl grer, fertig
	cp TEMP3,TEMP5    ; MSB gleich, vergleiche LSBs
	brlo RECHTS       ; LSB Binrzahl kleiner, fertig
	rjmp LINKS
	
AZLO:   ;Soll kleiner Ist
	sub TEMP5,TEMP3    ; Subtrahiere LSB Dezimalzahl
	sbc TEMP6,TEMP4    ; Subtrahiere Carry und MSB
	
	mov TEMP3,Hystlo     ; Abweichung * 18
	mov TEMP4,Hysthi     ; Abweichung * 18
	
	cp TEMP6,TEMP4    ; Vergleiche MSBs miteinander
	brlo STOP   	  ; MSB Binrzahl kleiner, fertig
	brne AZLOW        ; MSB Binrzahl grer, fertig
	cp TEMP5,TEMP3    ; MSB gleich, vergleiche LSBs
	brlo STOP         ; LSB Binrzahl kleiner, fertig
	breq STOP

AZLOW:	ldi TEMP3, LOW(1800*18)
	ldi TEMP4, HIGH(1800*18)
	
	cp TEMP6,TEMP4    ; Vergleiche MSBs miteinander
	brlo LINKS   	  ; MSB Binrzahl kleiner, fertig
	brne RECHTS       ; MSB Binrzahl grer, fertig
	cp TEMP5,TEMP3    ; MSB gleich, vergleiche LSBs
	brlo LINKS        ; LSB Binrzahl kleiner, fertig
	rjmp RECHTS	

STOP:
	;clr TByteCNT	     ; Initialisiere Bytezhler
	;ldi ZH, HIGH(2*AXR)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse
        ;ldi ZL, LOW(2*AXR)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse
        ;sbic PortC, RotateRight
        ;rcall Transmit
	
	cbi PortC, RotateRight
	cbi PortC, RotateLeft
	;TXLum
	rjmp Main

RECHTS:
	sbi PortC, RotateRight
	cbi PortC, RotateLeft
	;TXLum
	rjmp Main

LINKS:
	sbi PortC, RotateRight
	sbi PortC, RotateLeft
	;TXLum
	rjmp Main
	
         

;******************** Hauptprogramm Ende ***********************************

AMsgOK:  clr TByteCNT	      ; Initialisiere Bytezhler
	 ldi ZH, HIGH(2*AXM)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse
         ldi ZL, LOW(2*AXM)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse
         rcall Transmit
AMsgEnd: ret
         
AGetSH:  clr TByteCNT	          ; Initialisiere Bytezhler
	 sbic PinD,ShutOpen
         rjmp RGetSH
         ldi ZH, HIGH(2*AXO)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse
         ldi ZL, LOW(2*AXO)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse
         rcall Transmit
         cbr FlagReg, (1<<GetSH)
         ret
RGetSH:  sbic PinD,ShutClosed
         rjmp IGetSH
         ldi ZH, HIGH(2*AXC)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse
         ldi ZL, LOW(2*AXC)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse
         rcall Transmit
         cbr FlagReg, (1<<GetSH)
         ret
IGetSH:  ldi ZH, HIGH(2*AXI)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse
         ldi ZL, LOW(2*AXI)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse
         rcall Transmit
         cbr FlagReg, (1<<GetSH)
         ret
         
AOpSH:   sbis PortC, OpenShut
	 rcall AMsgOK
	 sbi PortC, OpenShut
	 cbi PortC, CloseShut
	 cbr FlagReg, (1<<ClSH)
	 
	 sbic PinD, ShutOpen
	 rjmp ROpSH
	 cbi PortC, OpenShut
	 cbi PortC, CloseShut
	 clr TByteCNT	      ; Initialisiere Bytezhler
	 ldi ZH, HIGH(2*AXO)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse
         ldi ZL, LOW(2*AXO)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse
         rcall Transmit
         cbr FlagReg, (1<<OpSH)
ROpSH:   ret
         
AClSH:   sbis PortC, OpenShut
	 rcall AMsgOK
	 sbi PortC, CloseShut
	 sbi PortC, OpenShut
	 cbr FlagReg, (1<<OpSH)
	 
	 sbic PinD, ShutClosed
	 rjmp RClSH
	 cbi PortC, CloseShut
	 cbi PortC, OpenShut
	 clr TByteCNT	      ; Initialisiere Bytezhler
	 ldi ZH, HIGH(2*AXC)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse
         ldi ZL, LOW(2*AXC)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse
         rcall Transmit
         cbr FlagReg, (1<<ClSH)
RClSH:   ret
         
Azisend: clr TByteCNT	       ; Initialisiere Bytezhler
	 ldi ZH, HIGH(2*AXAZ)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse
         ldi ZL, LOW(2*AXAZ)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse
         rcall Transaz
         ;sbis PinB, LongAZI    ; Wenn SCK auf Null dann sende langen Azimut
         ;rcall Transazval
         ;sbis PinB, StartVAL   ; Wenn MOSI auf Null dann sende Startwerte
         ;rcall Transstartval
	 cbr FlagReg, (1<<GetAz)
         ret
         
AValid:  
	 ldi XH, HIGH(RAMRX)  ; Inizialisiere X Pointer High-Adresse mit RAM-Start-Adresse ( Datenempfang )
         ldi XL, LOW(RAMRX)   ; Inizialisiere X Pointer Low-Adresse mit RAM-Start-Adresse ( Datenempfang )
         ldi ZH, HIGH(2*Commands)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse ( Datenempfang )
         ldi ZL, LOW(2*Commands)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse ( Datenempfang )
         clr TByteCNT	      ; Initialisiere Bytezhler
                 
CP1:     lpm		      ; Hole String
         ld DATA, X	      ; Hole Daten
                 	
         cp DATA, STRING      ; Vergleich 	
         brne CP2Z	      ; wenn ungleich dann nchster String	
         adiw XH:XL, 1	      ; inc X Pointer	
	 adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, RXLEN  ; Vergleiche auf Telegrammende
	 breq getshut	      ; und springe zur Auswertung
	 rjmp CP1	      ; Schleife
	 
getshut: sbr FlagReg, (1<<GetSH)
	 cbr Flagreg, (1<<TValid)
	 ret
	 
CP2Z:    adiw ZH:ZL, SLEN     ; Add Stringlnge zu Z Pointer
CP2:	 lpm		      ; Hole String
         ld DATA, X	      ; Hole Daten	
                  
         cp DATA, STRING      ; Vergleich 	
         brne CP3Z	      ; wenn ungleich dann nchster String	
         adiw XH:XL, 1	      ; inc X Pointer	
	 adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, RXLEN  ; Vergleiche auf Telegrammende
	 breq opeshut	      ; und springe zur Auswertung
	 rjmp CP2	      ; Schleife

opeshut: sbr FlagReg, (1<<OpSH)
	 cbr Flagreg, (1<<ClSH)
	 cbr Flagreg, (1<<TValid)
	 
	 ret
	 
CP3Z:    adiw ZH:ZL, SLEN     ; Add Stringlnge zu Z Pointer
CP3:	 lpm		      ; Hole String
         ld DATA, X	      ; Hole Daten	
                         
         cp DATA, STRING      ; Vergleich 	
         brne CP4Z	      ; wenn ungleich dann nchster String	
         adiw XH:XL, 1	      ; inc X Pointer	
	 adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, RXLEN  ; Vergleiche auf Telegrammende
	 breq closhut	      ; und springe zur Auswertung
	 rjmp CP3	      ; Schleife	 

closhut: sbr FlagReg, (1<<ClSH)
	 cbr Flagreg, (1<<OpSH)
	 cbr Flagreg, (1<<TValid)
	 ret
	 
CP4Z:    adiw ZH:ZL, SLEN     ; Add Stringlnge zu Z Pointer
CP4:	 lpm		      ; Hole String
         ld DATA, X	      ; Hole Daten	
                          
         cp DATA, STRING      ; Vergleich 	
         brne CP7Z	      ; wenn ungleich dann nchster String	
         adiw XH:XL, 1	      ; inc X Pointer	
	 adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, RXLEN  ; Vergleiche auf Telegrammende
	 breq getazim	      ; und springe zur Auswertung
	 rjmp CP4	      ; Schleife	 

getazim: sbr FlagReg, (1<<GetAz)
	 cbr Flagreg, (1<<TValid)
	 ret
	 
CP7Z:    adiw ZH:ZL, SLEN     ; Add Stringlnge zu Z Pointer
CP7:	 lpm		      ; Hole String
         ld DATA, X	      ; Hole Daten	
                          
         cp DATA, STRING      ; Vergleich 	
         brne CP8Z	      ; wenn ungleich dann nchster String	
         adiw XH:XL, 1	      ; inc X Pointer	
	 adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, RXLEN  ; Vergleiche auf Telegrammende
	 breq getmult	      ; und springe zur Auswertung
	 rjmp CP7	      ; Schleife	 

getmult: sbr FlagRg2, (1<<GetMUL)
	 cbr Flagreg, (1<<TValid)
	 ret	 

CP8Z:    adiw ZH:ZL, SLEN     ; Add Stringlnge zu Z Pointer
CP8:	 lpm		      ; Hole String
         ld DATA, X	      ; Hole Daten	
                          
         cp DATA, STRING      ; Vergleich 	
         brne CP5Z	      ; wenn ungleich dann nchster String	
         adiw XH:XL, 1	      ; inc X Pointer	
	 adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, RXLEN  ; Vergleiche auf Telegrammende
	 breq gethyst	      ; und springe zur Auswertung
	 rjmp CP8	      ; Schleife	 

gethyst: sbr FlagRg2, (1<<GetHYS)
	 cbr Flagreg, (1<<TValid)
	 ret	 
	 
CP5Z:    adiw ZH:ZL, SLEN     ; Add Stringlnge zu Z Pointer
CP5:	 lpm		      ; Hole String
         ld DATA, X	      ; Hole Daten	
                  
         cp DATA, STRING      ; Vergleich 	
         brne CP9Z	      ; wenn ungleich Fehler	
         adiw XH:XL, 1	      ; inc X Pointer	
	 adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, AZILEN ; Vergleiche auf Telegrammende
	 breq CP6Z	      ; und springe zur Auswertung
	 rjmp CP5	      ; Schleife	 

CP6Z:	 ldi TEMP, LOW(10)   
	 ldi TEMP1, HIGH(10)  ; lade Multiplikator
	 clr TEMP2	      ; bereinige TEMP Register
	 clr TEMP3	      ; bereinige TEMP Register
	 clr TEMP4	      ; bereinige TEMP Register
	 clr TEMP5	      ; bereinige TEMP Register
	 clr TEMP6	      ; bereinige TEMP Register
	 clr TEMP7	      ; bereinige TEMP Register
	 clr TEMP8	      ; bereinige TEMP Register
	 
	 	 
CP6:	 ld DATA, X	      ; Hole Daten	
         subi DATA, 48	      ; Subtrahiere 48
         brmi Fault	      ; wenn minus Fehler
         cpi DATA, 10	      ; vergleiche mit 10
         brsh Fault	      ; wenn grer/gleich Fehler
         
         add TEMP2, DATA      ; addiere Daten	
         adc TEMP3, TEMP8     ;	addiere NULL und Carry
         
         adiw XH:XL, 1	      ; inc X Pointer       	 
         inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, RXLEN  ; Vergleiche auf Telegrammende
	 breq azi	      ; und springe zur Auswertung
	 
	 rcall mpy16u	      ; multipliziere mit 10
	 
	 rjmp CP6	      ; Schleife
	 
Fault:   cbr Flagreg, (1<<TValid)
	 ret
	 
azi:     ;Test auf Zahlenwert 3600 = 0E10

	 cpi TEMP3, 0x0E      ; teste HI-Byte
	 breq RangeEND   	     
	 brlo AZOK
	 rjmp Fault
	 
RangeEND:cpi TEMP2, 0x10	 
	 brsh Fault
	 
AZOK:	 rcall AMsgOK
	 
	 ldi TEMP, LOW(18)   
	 ldi TEMP1, HIGH(18)
	 
	 rcall mpy16u	      ; multipliziere mit 18
	 	
	 mov AziLO, TEMP2     ; sichere Ergebnis
	 mov AziHI, TEMP3     ; sichere Ergebnis
	 
	 clr TByteCNT	       ; Initialisiere Bytezhler
	 ldi ZH, HIGH(2*AXAZ)  ; Inizialisiere Z Pointer High-Adresse mit LPM-Start-Adresse
         ldi ZL, LOW(2*AXAZ)   ; Inizialisiere Z Pointer Low-Adresse mit LPM-Start-Adresse
         ;rcall TransVAL
         	          
	 cbr Flagreg, (1<<TValid)
	 ret ;empfangener Azi OK
	 
CP9Z:    adiw ZH:ZL, SLEN     ; Add Stringlnge zu Z Pointer
CP9:	 lpm		      ; Hole String
         ld DATA, X	      ; Hole Daten	
         TXlum         
         cp DATA, STRING      ; Vergleich 	
         brne CP11Z	      ; wenn ungleich dann nchster String	
         adiw XH:XL, 1	      ; inc X Pointer	
	 adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, AZILEN ; Vergleiche auf Telegrammende
	 breq CP10Z	      ; und springe zur Auswertung
	 rjmp CP9	      ; Schleife	 

CP10Z:	 ldi TEMP, LOW(10)   
	 ldi TEMP1, HIGH(10)  ; lade Multiplikator
	 clr TEMP2	      ; bereinige TEMP Register
	 clr TEMP3	      ; bereinige TEMP Register
	 clr TEMP4	      ; bereinige TEMP Register
	 clr TEMP5	      ; bereinige TEMP Register
	 clr TEMP6	      ; bereinige TEMP Register
	 clr TEMP7	      ; bereinige TEMP Register
	 clr TEMP8	      ; bereinige TEMP Register
	 
	 	 
CP10:	 ld DATA, X	      ; Hole Daten	
         subi DATA, 48	      ; Subtrahiere 48
         brmi Fault10	      ; wenn minus Fehler
         cpi DATA, 10	      ; vergleiche mit 10
         brsh Fault10	      ; wenn grer/gleich Fehler
         
         add TEMP2, DATA      ; addiere Daten	
         adc TEMP3, TEMP8     ;	addiere NULL und Carry
         
         adiw XH:XL, 1	      ; inc X Pointer       	 
         inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, RXLEN  ; Vergleiche auf Telegrammende
	 breq MUL	      ; und springe zur Auswertung
	 
	 rcall mpy16u	      ; multipliziere mit 10
	 
	 rjmp CP10	      ; Schleife
	 
Fault10: cbr Flagreg, (1<<TValid)
	 ret
	 
MUL:     ;Test auf Zahlenwert 3600 = 0E10
	 RXlum
	 cpi TEMP3, 0x0E      ; teste HI-Byte
	 breq MULEND   	     
	 brlo MULOK
	 rjmp Fault10
	 
MULEND:  cpi TEMP2, 0x10	 
	 brsh Fault10
	 
MULOK:	 mov TEMP1, TEMP2     ; sichere Ergebnis LoByte
	 ldi TEMP2, 1	      ; Adresse LoByte
	 rcall WriteEEprom
	 	
	 mov TEMP1, TEMP3     ; sichere Ergebnis HiByte
	 ldi TEMP2, 0	      ; Adresse HiByte
	 rcall WriteEEprom
	 
	 rcall AMsgOK	          
	 cbr Flagreg, (1<<TValid)
	 
	 ;Teiler aus EEPROM laden

         ldi TEMP , 0	      ; Teileradresse
         rcall readEEPROM
         mov TeilerHI, TEMP1
         
         ldi TEMP , 1	      ; Teileradresse
         rcall readEEPROM
         mov TeilerLO, TEMP1
	 ret	 	                                    

CP11Z:   adiw ZH:ZL, SLEN     ; Add Stringlnge zu Z Pointer
CP11:	 lpm		      ; Hole String
         ld DATA, X	      ; Hole Daten	
         TXlum         
         cp DATA, STRING      ; Vergleich 	
         brne Fault12	      ; wenn ungleich Fehler	
         adiw XH:XL, 1	      ; inc X Pointer	
	 adiw ZH:ZL, 1	      ; inc Z Pointer
	 inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, AZILEN ; Vergleiche auf Telegrammende
	 breq CP12Z	      ; und springe zur Auswertung
	 rjmp CP11	      ; Schleife	 

CP12Z:	 ldi TEMP, LOW(10)   
	 ldi TEMP1, HIGH(10)  ; lade Multiplikator
	 clr TEMP2	      ; bereinige TEMP Register
	 clr TEMP3	      ; bereinige TEMP Register
	 clr TEMP4	      ; bereinige TEMP Register
	 clr TEMP5	      ; bereinige TEMP Register
	 clr TEMP6	      ; bereinige TEMP Register
	 clr TEMP7	      ; bereinige TEMP Register
	 clr TEMP8	      ; bereinige TEMP Register
	 
	 	 
CP12:	 ld DATA, X	      ; Hole Daten	
         subi DATA, 48	      ; Subtrahiere 48
         brmi Fault12	      ; wenn minus Fehler
         cpi DATA, 10	      ; vergleiche mit 10
         brsh Fault12	      ; wenn grer/gleich Fehler
         
         add TEMP2, DATA      ; addiere Daten	
         adc TEMP3, TEMP8     ;	addiere NULL und Carry
         
         adiw XH:XL, 1	      ; inc X Pointer       	 
         inc TByteCNT         ; inc Zhler	
	 cpi TByteCNT, RXLEN  ; Vergleiche auf Telegrammende
	 breq HYS	      ; und springe zur Auswertung
	 
	 rcall mpy16u	      ; multipliziere mit 10
	 
	 rjmp CP12	      ; Schleife
	 
Fault12: cbr Flagreg, (1<<TValid)
	 ret
	 
HYS:     ;Test auf Zahlenwert 3600 = 0E10
	 RXlum
	 cpi TEMP3, 0x0E      ; teste HI-Byte
	 breq HYSEND   	     
	 brlo HYSOK
	 rjmp Fault12
	 
HYSEND:  cpi TEMP2, 0x10	 
	 brsh Fault12
	 
HYSOK:	 ldi TEMP, LOW(18)   
	 ldi TEMP1, HIGH(18)
	 
	 rcall mpy16u	      ; multipliziere mit 18
	 	
	 mov TEMP1, TEMP2     ; sichere Ergebnis LoByte
	 ldi TEMP2, 3	      ; Adresse LoByte
	 rcall WriteEEprom
	 	
	 mov TEMP1, TEMP3     ; sichere Ergebnis HiByte
	 ldi TEMP2, 2	      ; Adresse HiByte
	 rcall WriteEEprom
	 
	 rcall AMsgOK	          
	 cbr Flagreg, (1<<TValid)
	 
	 ;Hysterse aus EEPROM laden

         ldi TEMP , 2	      ; HystereseAdresse
         rcall readEEPROM
         mov HYSTHI, TEMP1
         
         ldi TEMP , 3	      ; HystereseAdresse
         rcall readEEPROM
         mov HYSTLO, TEMP1
         ret 

	 	                                    
	 
Commands:

;Commands from Telecope
.db "getshut",1;(Get Shutterstatus)      64,23, 67,65,74,73,68,75,74
.db "opeshut",2;(open shutter)           64,23, 6F,70,65,73,68,75,74
.db "closhut",3;(schliee Kuppelspalt)   64,23, 63,6C,6F,73,68,75,74
.db "getazim",4;(get Azimutposition)     64,23, 67,65,74,61,7A,69,6D
.db "getmult",7;(get Multiplikator)
.db "gethyst",8;(get Hysterese)
.db "azi0000",5;(South)                  64,23, 61,7A,69,30,30,30,30
.db "mul0000",9;(Teilerwert)             
.db "hys0000",9;(Hysteresewert)


;Answers to Telescope
AXO:
.db "d#shutope",0;(if shutter is opened)
AXC:
.db "d#shutclo",0;(if shutter is closed)
AXI:
.db "d#shutinb",0;(if shutter is moving / inbetween)
AXM:
.db "d#gotmess",0;answer for correkt receive

;db "d#shutope",0;After the shutter is opened completely, the dome sends
;db "d#shutclo",0;After the shutter is closed completely, the dome sends
;db "d#azi0000",0;(South)
;db "d#azi0900",0;(West)
AXR:
.db "d#reached",0;After the position is reached, the dome sends back:
AXAZ:
.db "d#azi",0;(North)in 1/10 degree steps (1800=180 degree)
AXMUL: 
.db "d#mul",0;Multiplyer dezimal 
AXHYS:
.db "d#hys",0;Hysteresis 1/10 degree steps
 


;********************ENDE des CODE Segments ********************************

; EEPROM beginnt hier
;
.ESEG
.ORG $0000
.DB 0x00, 0x57 ;Teiler HiByte / LoByte
.DB 0x00, 0xB4 ;Hysterese HiByte / LoByte


